#ifndef __PCIE40_DRIVER_COMMON_H
#define __PCIE40_DRIVER_COMMON_H
//p40driver``+

#include <linux/cdev.h>
#include <linux/device.h>
#include <linux/kernel.h>
#include <linux/list.h>
#include <linux/module.h>
#include <linux/pci.h>
#include <linux/version.h>

#include "altera_dma_regs.h"
#ifndef P40_FMT
#error "#define P40_FMT before including this file"
#endif
#define P40_DRV_NAME "pcie40"
#define P40_INFO KERN_INFO P40_FMT
#define P40_WARN KERN_WARNING P40_FMT
#define P40_DIAG KERN_DEBUG P40_FMT
#define P40_ERR  KERN_ERR P40_FMT
#define P40_PARM __FUNCTION__

#define P40_MAX_BAR 5
#define BAR0 0
#define BAR2 2
#define BAR4 4
#define P40_COMMON_BARS_MASK ((1<<1)|(1<<2)) //BAR1

#define PCI_MAX_ALLOC (4*1024*1024) //0x20000

//+`pcie40_state`
struct pcie40_state {
  struct list_head list;
  struct pci_dev *pci_dev;

  unsigned long bar_start[P40_MAX_BAR];
  unsigned long bar_size[P40_MAX_BAR];

  int link_id; //0 or 1
  int dev_id; //unique device identifier, even for all link 0s, odd for all link 1s


  void __iomem *bar0_regs;
  void __iomem *bar1_regs;
  void __iomem *bar2_regs;
  void __iomem *bar4_regs;



  struct pcie40_ecs_state *ecs_state;
  struct pcie40_dma_state *dma_state;
  struct pcie40_cvp_state *cvp_state;
};

static inline uint32_t pcie40_read32_bar0(struct pcie40_state *common, unsigned long offset)
{

  return ioread32((u32*)(common->bar0_regs) + offset);

}

static inline void pcie40_write32_bar0(struct pcie40_state *common, unsigned long offset, uint32_t value)
{

  iowrite32(value, (u32*)(common->bar0_regs) + offset);

}

static inline uint32_t pcie40_read32_bar2(struct pcie40_state *common, unsigned long offset)
{
  uint32_t val = ioread32((u32*)(common->bar2_regs) + offset);
 rmb();
 
 return val;

}

static inline void pcie40_write32_bar2(struct pcie40_state *common, unsigned long offset, uint32_t value)
{

  iowrite32(value, (u32*)(common->bar2_regs) + offset);

}
static inline uint32_t pcie40_wait_data(struct pcie40_state *common)
{
  uint32_t val=0;
  u32	timeout=0x20;
  while(1) {
    val = ioread32((u32*)(common->bar2_regs) +ALTERA_STATUS_FIFO_DATA);
    rmb();
   if  ( (val & 0x100)== 0x100 ) return 1;  
   timeout--;
   // cpu_relax(); fait planter le DMA si l'on passe là !! et reboot pour remettre en etat
   if(timeout == 0)  return 0;
  
   }
 return 0;

}

static inline void pcie40_reset_fifo(struct pcie40_state *common)
{
  uint32_t val;
   val = ioread32((u32*)(common->bar2_regs) +ALTERA_RESET_FIFO_DATA);
  val = val & 0x100;
     iowrite32(val, (u32*)(common->bar2_regs) + ALTERA_RESET_FIFO_DATA);
  val = val | 0xFEFF ;
  iowrite32(val, (u32*)(common->bar2_regs) + ALTERA_RESET_FIFO_DATA);
}

static inline int pcie40_device_accessible(struct pcie40_state *common)
{
  //uint32_t version = pcie40_read32_ctrl(common, P40_DMA_CTRL_OFF_VERSION);
  uint32_t version=0;
  return version != 0xFFFFFFFF;
}


static inline char *pcie40_devnode(struct device *dev, umode_t *mode)
{
  if (mode) {
    *mode = 0666;
  }
  return NULL;
}

int pcie40_setup_cdev(struct class *cls, struct cdev *cdev, dev_t dev_num, int minor, int bar, const char *dev_name, int dev_id, struct file_operations *fops);

#endif//__PCIE40_DRIVER_COMMON_H
